require File.expand_path('../../test_helper', __FILE__)

class RcrmActsAsDraftableTest < ActiveSupport::TestCase
  def test_rcrm_acts_as_draftable_without_arguments
    assert_nothing_raised do
      Cell.rcrm_acts_as_draftable
    end
  end

  class SomeClass < ActiveRecord::Base; end
  def test_rcrm_acts_as_draftable_with_parent
    assert_nothing_raised do
      News.rcrm_acts_as_draftable(parent: :author)
    end
    assert User.new.respond_to?(:drafts)
  end

  def test_rcrm_acts_as_draftable_with_non_existing_parent
    assert_raises do
      Issue.rcrm_acts_as_draftable(parent: :foo)
    end
  end

  def test_rcrm_acts_as_draftable_with_non_hash_argument
    assert_raises do
      Issue.rcrm_acts_as_draftable('bar')
    end
  end

  def test_rcrm_acts_as_draftable_with_invalid_hash_key
    assert_raises do
      Issue.rcrm_acts_as_draftable(baz: 'qux')
    end
  end

  def test_drafts
    issue = Issue.new
    issue.save_draft(current_user)

    assert_equal 1, Issue.drafts(current_user).count
    assert_equal YeshouaCrm::ActsAsDraftable::Draft, Issue.drafts(current_user).first.class
  end

  def test_from_draft
    issue = Issue.new(subject: 'subject')
    issue.save_draft(current_user)

    issue_from_draft = Issue.from_draft(issue.draft_id)

    assert_equal Issue, issue_from_draft.class
    assert_equal true, issue_from_draft.new_record?
    assert_equal issue.draft_id, issue_from_draft.draft_id

    assert_equal issue.subject, issue_from_draft.subject
  end

  def test_draft_deleted_after_save
    issue = Issue.new(subject: 'subject')
    issue.save_draft(current_user)

    issue_from_draft = Issue.from_draft(issue.draft_id)

    assert_difference 'Issue.count', 1 do
      assert_difference 'YeshouaCrm::ActsAsDraftable::Draft.count', -1 do
        issue_from_draft.save!
      end
    end
    assert_nil issue_from_draft.draft_id
  end

  def test_from_draft_with_non_existing_draft_id
    assert_raises do
      Issue.from_draft(999)
    end
  end

  def test_from_draft_with_wrong_type
    Cell.rcrm_acts_as_draftable

    cell = Cell.new
    cell.save_draft(current_user)

    assert_raises do
      Issue.from_draft(cell.draft_id)
    end
  end

  def test_dump_to_draft
    assert_equal String, Issue.new.dump_to_draft.class
  end

  def test_load_to_draft
    attributes = {subject: 'subject', description: 'description'}
    draft = Issue.new(attributes).dump_to_draft

    issue = Issue.new
    issue.load_from_draft(draft)

    assert_equal attributes[:subject], issue.subject
    assert_equal attributes[:description], issue.description
  end

  def test_save_draft
    attributes = {subject: 'subject'}
    issue = Issue.new(attributes)

    assert_difference 'Issue.count', 0 do
      assert_difference 'YeshouaCrm::ActsAsDraftable::Draft.count', 1 do
        result = issue.save_draft(current_user)
        assert_equal true, result
      end
    end
    assert_equal Fixnum, issue.draft_id.class


    draft = YeshouaCrm::ActsAsDraftable::Draft.find(issue.draft_id)
    assert_equal 'Issue', draft.target_type
    assert_equal current_user.id, draft.user_id
    assert_equal attributes[:subject], draft.restore.subject
  end

  def test_save_draft_without_user
    issue = Issue.new

    assert_difference 'Issue.count', 0 do
      assert_difference 'YeshouaCrm::ActsAsDraftable::Draft.count', 1 do
        result = issue.save_draft
        assert_equal true, result
      end
    end

    draft = YeshouaCrm::ActsAsDraftable::Draft.find(issue.draft_id)
    assert_nil draft.user_id
  end

  def test_save_draft_with_associations
    issue = Issue.new(cell: cells(:second_cell))
    issue.save_draft(current_user)

    draft = YeshouaCrm::ActsAsDraftable::Draft.find(issue.draft_id)
    assert_equal issue.cell.name, draft.restore.cell.name
  end

  def test_save_draft_updates_existing_draft
    issue = Issue.new
    issue.save_draft(current_user)

    issue.subject = 'changed subject'
    assert_difference 'Issue.count', 0 do
      assert_difference 'YeshouaCrm::ActsAsDraftable::Draft.count', 0 do
        issue.save_draft(current_user)
      end
    end

    draft = YeshouaCrm::ActsAsDraftable::Draft.find(issue.draft_id)
    assert_equal issue.subject, draft.restore.subject
  end

  def test_save_draft_does_not_save_persisted_model
    issue = Issue.new
    issue.save_draft(current_user)
    issue.save!
    assert_equal false, issue.save_draft(current_user)
  end

  def test_update_draft
    issue = Issue.new(subject: 'subject')
    issue.save_draft(current_user)

    assert_difference 'Issue.count', 0 do
      assert_difference 'YeshouaCrm::ActsAsDraftable::Draft.count', 0 do
        issue.update_draft(current_user, subject: 'updated_subject')
      end
    end

    draft = YeshouaCrm::ActsAsDraftable::Draft.find(issue.draft_id)
    assert_equal issue.subject, draft.restore.subject
  end

  private

  def current_user
    users(:sam)
  end
end
